#import "RValue.h"
#import "REnvironment.h"
#import "RParsedExpression.h"
#import "RSmartLock.h"

#include <Rinternals.h>
#include <R_ext/Parse.h>

@implementation RValue

+ (NSLock*)gil {
	static NSLock *globalInterpreterLock = nil;
	if(nil == globalInterpreterLock)
		globalInterpreterLock = [[NSLock alloc] init];
	return globalInterpreterLock;
}
+ (RValue*)valueWithPointer:(void*)aPtr {
	return [[[self alloc] initWithPointer:aPtr] autorelease];
}

+ (RValue*)valueWithString:(NSString*)aStr {
	SEXP str;
	PROTECT(str = allocVector(STRSXP,1));
	SET_STRING_ELT(str,0,mkChar([aStr UTF8String]));
	RValue *rval = [self valueWithPointer:str];
	UNPROTECT(1);
	return rval;
}
+ (RValue*)valueWithInteger:(NSInteger)anInt {
	return [self valueWithPointer:ScalarInteger(anInt)];
}

- (RParsedExpression*)parseWithError:(NSError**)error {
	SEXP parsedValue;
	if(error != NULL) *error = nil;
	ParseStatus status = PARSE_NULL;
//	Rf_PrintValue((SEXP)rPtr);
	parsedValue = R_ParseVector((SEXP)rPtr,length((SEXP)rPtr),&status,R_NilValue);
	switch(status) {
	case PARSE_OK:
//		Rf_PrintValue(parsedValue);
		return (RParsedExpression*)[RParsedExpression valueWithPointer:parsedValue];
	default:
		if(error != NULL) *error = [NSError errorWithDomain:@"RValueParseError" code:status userInfo:nil];
		break;
	}
	return nil;
}

- (id)initWithPointer:(void*)aPtr {
	if(nil == [super init]) return nil;
	rPtr = aPtr;
	R_PreserveObject(rPtr);
	return self;
}

- (void)finalize {
	R_ReleaseObject(rPtr);
	rPtr = NULL;
	[super finalize];
}


- (RValue*)evaluateInEnvironment:(REnvironment*)env block:(BOOL)shouldBlock {
	return [env evaluate:self blocking:shouldBlock];
}

- (RValue*)evaluate {
	return [self evaluateInEnvironment:[REnvironment global] block:YES];
}

- (void*)pointer { return rPtr; }

- (NSInteger)length { return length(rPtr); }

- (NSString*)stringValue {
	switch(TYPEOF(rPtr)) {
		case STRSXP:
			return [NSString stringWithCString:CHAR(STRING_ELT(rPtr,0))];
		default:
		break;
	}
	return @"";
}

- (NSArray*)arrayValue {
	NSMutableArray *arr;
	int i;
	switch(TYPEOF(rPtr)) {
	case STRSXP:
		arr = [[NSMutableArray alloc] init];
		for(i=0;i<length(rPtr);i++) [arr addObject:[NSString stringWithCString:CHAR(STRING_ELT(rPtr,i))]];
		return arr;
	default:
		break;
	}
	return nil;
}


- (NSString*)description {
	switch(TYPEOF(rPtr)) {
		case NILSXP:
			return @"RValue<NULL>";
		case STRSXP:
			return [NSString stringWithFormat:@"RValue<String:\"%@\">",[self stringValue]];
		default:
			return [NSString stringWithFormat:@"Rvalue<%ld>",(long)TYPEOF(rPtr)];
		break;
	}
}

- (BOOL)isStringType { return (TYPEOF(rPtr) == STRSXP) ? YES : NO; }

@end
